@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999
+ * Copyright (c) 1998, 1999
  *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001
+ * Copyright (c) 1999, 2000, 2001, 2002
  *             Tama Communications Corporation. All rights reserved.
  *
  * This file is part of GNU GLOBAL.
@@ -43,10 +43,12 @@
 #include "defined.h"
 #include "die.h"
 #include "strbuf.h"
+#include "strlimcpy.h"
 #include "token.h"
 
 static	int	function_definition(int);
 static	void	condition_macro(int);
+static	int	seems_datatype(const char *);
 static	int	reserved(char *);
 
 /*
@@ -111,20 +113,27 @@ C(yacc)
 					char	savetok[MAXTOKEN], *saveline;
 					int	savelineno = lineno;
 
-					strcpy(savetok, token);
+					strlimcpy(savetok, token, sizeof(savetok));
 					strbuf_reset(sb);
 					strbuf_puts(sb, sp);
 					saveline = strbuf_value(sb);
-					if (function_definition(target))
+					if (function_definition(target)) {
 						if (target == DEF)
 							PUT(savetok, savelineno, saveline);
+					} else {
+						if (target == REF && defined(savetok))
+							PUT(savetok, savelineno, saveline);
+					}
 				}
 			} else {
 				if (dflag) {
-					if (target == REF && defined(token))
-						PUT(token, lineno, sp);
-					else if (target == SYM && !defined(token))
-						PUT(token, lineno, sp);
+					if (target == REF) {
+						if (defined(token))
+							PUT(token, lineno, sp);
+					} else if (target == SYM) {
+						if (!defined(token))
+							PUT(token, lineno, sp);
+					}
 				} else {
 					if (target == SYM)
 						PUT(token, lineno, sp);
@@ -287,6 +296,139 @@ C(yacc)
 			if (wflag && !startmacro && level == 0)
 				fprintf(stderr, "Warning: Out of function. %8s [+%d %s]\n", token, lineno, curfile);
 			break;
+		case C_TYPEDEF:
+			if (tflag) {
+				char	savetok[MAXTOKEN];
+				int	savelineno = 0;
+				int	typedef_savelevel = level;
+
+				savetok[0] = 0;
+				c = nexttoken("{}(),;", reserved);
+				if (wflag && c == EOF) {
+					fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+					break;
+				} else if (c == C_ENUM || c == C_STRUCT || c == C_UNION) {
+					char *interest_enum = "{},;";
+					int c_ = c;
+
+					c = nexttoken(interest_enum, reserved);
+					/* read enum name if exist */
+					if (c == SYMBOL) {
+						if (target == SYM)
+							PUT(token, lineno, sp);
+						c = nexttoken(interest_enum, reserved);
+					}
+					for (; c != EOF; c = nexttoken(interest_enum, reserved)) {
+						switch (c) {
+						case CP_IFDEF:
+						case CP_IFNDEF:
+						case CP_IF:
+						case CP_ELIF:
+						case CP_ELSE:
+						case CP_ENDIF:
+						case CP_UNDEF:
+							condition_macro(c);
+							continue;
+						default:
+							break;
+						}
+						if (c == ';' && level == typedef_savelevel) {
+							if (savetok[0] && target == DEF)
+								PUT(savetok, savelineno, sp);
+							break;
+						} else if (c == '{')
+							level++;
+						else if (c == '}') {
+							if (--level == typedef_savelevel)
+								break;
+						} else if (c == SYMBOL) {
+							if (c_ == C_ENUM) {
+								if (target == DEF && level > typedef_savelevel)
+									PUT(token, lineno, sp);
+								if (target == SYM && level == typedef_savelevel)
+									PUT(token, lineno, sp);
+							} else {
+								if (target == REF) {
+									if (level > typedef_savelevel && defined(token))
+										PUT(token, lineno, sp);
+								} else if (target == SYM) {
+									if (!defined(token))
+										PUT(token, lineno, sp);
+								} else if (target == DEF) {
+									/* save lastest token */
+									strlimcpy(savetok, token, sizeof(savetok));
+									savelineno = lineno;
+								}
+							}
+						}
+					}
+					if (c == ';')
+						break;
+					if (wflag && c == EOF) {
+						fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+						break;
+					}
+					if (wflag && level != typedef_savelevel || c != '}') {
+						fprintf(stderr, "Warning: uneven {}. [+%d %s]\n", lineno, curfile);
+						break;
+					}
+				} else if (c == SYMBOL) {
+					if (target == REF && defined(token))
+						PUT(token, lineno, sp);
+					if (target == SYM && !defined(token))
+						PUT(token, lineno, sp);
+				}
+				savetok[0] = 0;
+				while ((c = nexttoken("(),;", reserved)) != EOF) {
+					switch (c) {
+					case CP_IFDEF:
+					case CP_IFNDEF:
+					case CP_IF:
+					case CP_ELIF:
+					case CP_ELSE:
+					case CP_ENDIF:
+					case CP_UNDEF:
+						condition_macro(c);
+						continue;
+					default:
+						break;
+					}
+					if (c == '(')
+						level++;
+					else if (c == ')')
+						level--;
+					else if (c == SYMBOL) {
+						if (level > 0) {
+							if (target == SYM)
+								PUT(token, lineno, sp);
+						} else {
+							/* put latest token if any */
+							if (savetok[0]) {
+								if (target == SYM)
+									PUT(savetok, savelineno, sp);
+							}
+							/* save lastest token */
+							strlimcpy(savetok, token, sizeof(savetok));
+							savelineno = lineno;
+						}
+					} else if (c == ',' || c == ';') {
+						if (savetok[0]) {
+							if (target == DEF)
+								PUT(savetok, lineno, sp);
+							savetok[0] = 0;
+						}
+					}
+					if (level == 0 && c == ';')
+						break;
+				}
+				if (wflag) {
+					if (c == EOF)
+						fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+					else if (level > 0)
+						fprintf(stderr, "Warning: () block unmatched. (last at level %d.)[+%d %s]\n", level, lineno, curfile);
+				}
+			}
+			break;
 		default:
 			break;
 		}
@@ -312,7 +454,7 @@ int	target;
 	int     brace_level, isdefine;
 
 	brace_level = isdefine = 0;
-	while ((c = nexttoken("(,)", reserved)) != EOF) {
+	while ((c = nexttoken("()", reserved)) != EOF) {
 		switch (c) {
 		case CP_IFDEF:
 		case CP_IFNDEF:
@@ -331,14 +473,20 @@ int	target;
 		else if (c == /* ( */')') {
 			if (--brace_level == 0)
 				break;
-		} else if (c == SYMBOL) {
-			if (target == SYM)
+		}
+		/* pick up symbol */
+		if (c == SYMBOL) {
+			if (target == REF) {
+				if (seems_datatype(token))
+					PUT(token, lineno, sp);
+			} else if (target == SYM)
 				PUT(token, lineno, sp);
 		}
 	}
 	if (c == EOF)
 		return 0;
-	while ((c = nexttoken(",;{}=", reserved)) != EOF) {
+	brace_level = 0;
+	while ((c = nexttoken(",;[](){}=", reserved)) != EOF) {
 		switch (c) {
 		case CP_IFDEF:
 		case CP_IFNDEF:
@@ -352,7 +500,11 @@ int	target;
 		default:
 			break;
 		}
-		if (c == SYMBOL || IS_RESERVED(c))
+		if (c == '('/* ) */ || c == '[')
+			brace_level++;
+		else if (c == /* ( */')' || c == ']')
+			brace_level--;
+		else if (brace_level == 0 && (c == SYMBOL || IS_RESERVED(c)))
 			isdefine = 1;
 		else if (c == ';' || c == ',') {
 			if (!isdefine)
@@ -360,10 +512,19 @@ int	target;
 		} else if (c == '{' /* } */) {
 			pushbacktoken();
 			return 1;
-		} else if (c == /* { */'}') {
+		} else if (c == /* { */'}')
 			break;
-		} else if (c == '=')
+		else if (c == '=')
 			break;
+
+		/* pick up symbol */
+		if (c == SYMBOL) {
+			if (target == REF) {
+				if (seems_datatype(token))
+					PUT(token, lineno, sp);
+			} else if (target == SYM)
+				PUT(token, lineno, sp);
+		}
 	}
 	return 0;
 }
@@ -421,6 +582,28 @@ condition_macro(cc)
 			}
 		}
 	}
+}
+/*
+ * seems_datatype: decide whether or not it is a data type.
+ *
+ *	i)	token	token
+ *	r)		0: not data type, 1: data type
+ */
+static int
+seems_datatype(token)
+const char *token;
+{
+	int length = strlen(token);
+	const char *p = token + length;
+
+	if (length < 3)
+		return 0;
+	if (strcmp(p - 2, "_t"))
+		return 1;
+	for (p = token; *p; p++)
+		if (islower(*p))
+			return 0;
+	return 1;
 }
 		/* sorted by alphabet */
 static struct words words[] = {
@@ -452,11 +635,13 @@ static struct words words[] = {
 	{"%{",		YACC_BEGIN},
 	{"%}",		YACC_END},
 	{"__P",		C___P},
+	{"__attribute__",C___ATTRIBUTE__},
 	{"asm",		C_ASM},
 	{"auto",	C_AUTO},
 	{"break",	C_BREAK},
 	{"case",	C_CASE},
 	{"char",	C_CHAR},
+	{"const",	C_CONST},
 	{"continue",	C_CONTINUE},
 	{"default",	C_DEFAULT},
 	{"do",		C_DO},
@@ -482,6 +667,8 @@ static struct words words[] = {
 	{"union",	C_UNION},
 	{"unsigned",	C_UNSIGNED},
 	{"void",	C_VOID},
+	{"volatile",	C_VOLATILE},
+	{"wchar_t",	C_WCHAR_T},
 	{"while",	C_WHILE},
 };
 
