@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999
+ * Copyright (c) 1998, 1999
  *             Shigio Yamaguchi. All rights reserved.
- * Copyright (c) 1999, 2000, 2001
+ * Copyright (c) 1999, 2000, 2001, 2002
  *             Tama Communications Corporation. All rights reserved.
  *
  * This file is part of GNU GLOBAL.
@@ -44,9 +44,11 @@
 #include "die.h"
 #include "locatestring.h"
 #include "strbuf.h"
+#include "strlimcpy.h"
 #include "token.h"
 
 static	int	function_definition(int);
+static  int     seems_datatype(const char *);
 static	void	condition_macro(int);
 static	int	reserved(char *);
 
@@ -115,7 +117,7 @@ Cpp()
 					char	savetok[MAXTOKEN], *saveline;
 					int	savelineno = lineno;
 
-					strcpy(savetok, token);
+					strlimcpy(savetok, token, sizeof(savetok));
 					strbuf_reset(sb);
 					strbuf_puts(sb, sp);
 					saveline = strbuf_value(sb);
@@ -123,20 +125,34 @@ Cpp()
 						/* ignore constructor */
 						if (target == DEF && strcmp(stack[classlevel].classname, savetok))
 							PUT(savetok, savelineno, saveline);
+					} else {
+						if (target == REF && defined(savetok))
+							PUT(savetok, savelineno, saveline);
 					}
 				}
 			} else {
-				if (target == SYM)
-					PUT(token, lineno, sp);
+				if (dflag) {
+					if (target == REF) {
+						if (defined(token))
+							PUT(token, lineno, sp);
+					} else if (target == SYM) {
+						if (!defined(token))
+							PUT(token, lineno, sp);
+					}
+				} else {
+					if (target == SYM)
+						PUT(token, lineno, sp);
+				}
 			}
 			break;
 		case CPP_CLASS:
 			DBG_PRINT(level, "class");
 			if ((c = nexttoken(interested, reserved)) == SYMBOL) {
-				strcpy(classname, token);
-				startclass = 1;
+				strlimcpy(classname, token, sizeof(classname));
 				if (target == DEF)
 					PUT(token, lineno, sp);
+				if (peekc(0) != ';')
+					startclass = 1;
 			}
 			break;
 		case '{':  /* } */
@@ -214,8 +230,13 @@ Cpp()
 				if (c == '\n')
 					pushbacktoken();
 			}  else {
-				if (target == SYM)
-					PUT(token, lineno, sp);
+				if (dflag) {
+					if (target == DEF)
+						PUT(token, lineno, sp);
+				} else {
+					if (target == SYM)
+						PUT(token, lineno, sp);
+				}
 			}
 			break;
 		case CP_INCLUDE:
@@ -320,6 +341,139 @@ Cpp()
 			if (wflag && !startmacro && level == 0)
 				fprintf(stderr, "Warning: Out of function. %8s [+%d %s]\n", token, lineno, curfile);
 			break;
+		case CPP_TYPEDEF:
+			if (tflag) {
+				char	savetok[MAXTOKEN];
+				int	savelineno = 0;
+				int	typedef_savelevel = level;
+
+				savetok[0] = 0;
+				c = nexttoken("{}(),;", reserved);
+				if (wflag && c == EOF) {
+					fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+					break;
+				} else if (c == CPP_ENUM || c == CPP_STRUCT || c == CPP_UNION) {
+					char *interest_enum = "{},;";
+					int c_ = c;
+
+					c = nexttoken(interest_enum, reserved);
+					/* read enum name if exist */
+					if (c == SYMBOL) {
+						if (target == SYM)
+							PUT(token, lineno, sp);
+						c = nexttoken(interest_enum, reserved);
+					}
+					for (; c != EOF; c = nexttoken(interest_enum, reserved)) {
+						switch (c) {
+						case CP_IFDEF:
+						case CP_IFNDEF:
+						case CP_IF:
+						case CP_ELIF:
+						case CP_ELSE:
+						case CP_ENDIF:
+						case CP_UNDEF:
+							condition_macro(c);
+							continue;
+						default:
+							break;
+						}
+						if (c == ';' && level == typedef_savelevel) {
+							if (savetok[0] && target == DEF)
+								PUT(savetok, savelineno, sp);
+							break;
+						} else if (c == '{')
+							level++;
+						else if (c == '}') {
+							if (--level == typedef_savelevel)
+								break;
+						} else if (c == SYMBOL) {
+							if (c_ == CPP_ENUM) {
+								if (target == DEF && level > typedef_savelevel)
+									PUT(token, lineno, sp);
+								if (target == SYM && level == typedef_savelevel)
+									PUT(token, lineno, sp);
+							} else {
+								if (target == REF) {
+									if (level > typedef_savelevel && defined(token))
+										PUT(token, lineno, sp);
+								} else if (target == SYM) {
+									if (!defined(token))
+										PUT(token, lineno, sp);
+								} else if (target == DEF) {
+									/* save lastest token */
+									strlimcpy(savetok, token, sizeof(savetok));
+									savelineno = lineno;
+								}
+							}
+						}
+					}
+					if (c == ';')
+						break;
+					if (wflag && c == EOF) {
+						fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+						break;
+					}
+					if (wflag && level != typedef_savelevel || c != '}') {
+						fprintf(stderr, "Warning: uneven {}. [+%d %s]\n", lineno, curfile);
+						break;
+					}
+				} else if (c == SYMBOL) {
+					if (target == REF && defined(token))
+						PUT(token, lineno, sp);
+					if (target == SYM && !defined(token))
+						PUT(token, lineno, sp);
+				}
+				savetok[0] = 0;
+				while ((c = nexttoken("(),;", reserved)) != EOF) {
+					switch (c) {
+					case CP_IFDEF:
+					case CP_IFNDEF:
+					case CP_IF:
+					case CP_ELIF:
+					case CP_ELSE:
+					case CP_ENDIF:
+					case CP_UNDEF:
+						condition_macro(c);
+						continue;
+					default:
+						break;
+					}
+					if (c == '(')
+						level++;
+					else if (c == ')')
+						level--;
+					else if (c == SYMBOL) {
+						if (level > 0) {
+							if (target == SYM)
+								PUT(token, lineno, sp);
+						} else {
+							/* put latest token if any */
+							if (savetok[0]) {
+								if (target == SYM)
+									PUT(savetok, savelineno, sp);
+							}
+							/* save lastest token */
+							strlimcpy(savetok, token, sizeof(savetok));
+							savelineno = lineno;
+						}
+					} else if (c == ',' || c == ';') {
+						if (savetok[0]) {
+							if (target == DEF)
+								PUT(savetok, lineno, sp);
+							savetok[0] = 0;
+						}
+					}
+					if (level == 0 && c == ';')
+						break;
+				}
+				if (wflag) {
+					if (c == EOF)
+						fprintf(stderr, "Warning: unexpected eof. [+%d %s]\n", lineno, curfile);
+					else if (level > 0)
+						fprintf(stderr, "Warning: () block unmatched. (last at level %d.)[+%d %s]\n", level, lineno, curfile);
+				}
+			}
+			break;
 		default:
 			break;
 		}
@@ -345,7 +499,7 @@ int	target;
 	int     brace_level, isdefine;
 
 	brace_level = isdefine = 0;
-	while ((c = nexttoken("(,)", reserved)) != EOF) {
+	while ((c = nexttoken("()", reserved)) != EOF) {
 		switch (c) {
 		case CP_IFDEF:
 		case CP_IFNDEF:
@@ -364,8 +518,13 @@ int	target;
 		else if (c == /* ( */')') {
 			if (--brace_level == 0)
 				break;
-		} else if (c == SYMBOL) {
-			if (target == SYM)
+		}
+		/* pick up symbol */
+		if (c == SYMBOL) {
+			if (target == REF) {
+				if (seems_datatype(token))
+					PUT(token, lineno, sp);
+			} else if (target == SYM)
 				PUT(token, lineno, sp);
 		}
 	}
@@ -375,7 +534,8 @@ int	target;
 		(void)nexttoken(";", NULL);
 		return 0;
 	}
-	while ((c = nexttoken(",;{}=", reserved)) != EOF) {
+	brace_level = 0;
+	while ((c = nexttoken(",;[](){}=", reserved)) != EOF) {
 		switch (c) {
 		case CP_IFDEF:
 		case CP_IFNDEF:
@@ -389,7 +549,11 @@ int	target;
 		default:
 			break;
 		}
-		if (c == SYMBOL || IS_RESERVED(c))
+		if (c == '('/* ) */ || c == '[')
+			brace_level++;
+		else if (c == /* ( */')' || c == ']')
+			brace_level--;
+		else if (brace_level == 0 && (c == SYMBOL || IS_RESERVED(c)))
 			isdefine = 1;
 		else if (c == ';' || c == ',') {
 			if (!isdefine)
@@ -397,10 +561,18 @@ int	target;
 		} else if (c == '{' /* } */) {
 			pushbacktoken();
 			return 1;
-		} else if (c == /* { */'}') {
+		} else if (c == /* { */'}')
 			break;
-		} else if (c == '=')
+		else if (c == '=')
 			break;
+		/* pick up symbol */
+		if (c == SYMBOL) {
+			if (target == REF) {
+				if (seems_datatype(token))
+					PUT(token, lineno, sp);
+			} else if (target == SYM)
+				PUT(token, lineno, sp);
+		}
 	}
 	return 0;
 }
@@ -458,6 +630,28 @@ condition_macro(cc)
 			}
 		}
 	}
+}
+/*
+ * seems_datatype: decide whether or not it is a data type.
+ *
+ *	i)	token	token
+ *	r)		0: not data type, 1: data type
+ */
+static int
+seems_datatype(token)
+const char *token;
+{
+	int length = strlen(token);
+	const char *p = token + length;
+
+	if (length < 3)
+		return 0;
+	if (strcmp(p - 2, "_t"))
+		return 1;
+	for (p = token; *p; p++)
+		if (islower(*p))
+			return 0;
+	return 1;
 }
 		/* sorted by alphabet */
 static struct words words[] = {
@@ -479,6 +673,7 @@ static struct words words[] = {
 	{"#warning",	CP_WARNING},
 	{"::",		CPP_SEP},
 	{"__P",		CPP___P},
+	{"__attribute__",CPP___ATTRIBUTE__},
 	{"asm",		CPP_ASM},
 	{"auto",	CPP_AUTO},
 	{"break",	CPP_BREAK},
@@ -494,6 +689,7 @@ static struct words words[] = {
 	{"double",	CPP_DOUBLE},
 	{"else",	CPP_ELSE},
 	{"enum",	CPP_ENUM},
+	{"explicit",	CPP_EXPLICIT},
 	{"extern",	CPP_EXTERN},
 	{"float",	CPP_FLOAT},
 	{"for",		CPP_FOR},
@@ -503,8 +699,11 @@ static struct words words[] = {
 	{"inline",	CPP_INLINE},
 	{"int",		CPP_INT},
 	{"long",	CPP_LONG},
+	{"mutable",	CPP_MUTABLE},
+	{"namespace",	CPP_NAMESPACE},
 	{"new",		CPP_NEW},
 	{"operator",	CPP_OPERATOR},
+	{"overload",	CPP_OVERLOAD},
 	{"private",	CPP_PRIVATE},
 	{"protected",	CPP_PROTECTED},
 	{"public",	CPP_PUBLIC},
@@ -521,8 +720,10 @@ static struct words words[] = {
 	{"throw",	CPP_THROW},
 	{"try",		CPP_TRY},
 	{"typedef",	CPP_TYPEDEF},
+	{"typename",	CPP_TYPENAME},
 	{"union",	CPP_UNION},
 	{"unsigned",	CPP_UNSIGNED},
+	{"using",	CPP_USING},
 	{"virtual",	CPP_VIRTUAL},
 	{"void",	CPP_VOID},
 	{"volatile",	CPP_VOLATILE},
